* Graphics for safety analysis
  Figure 2.5, Patient profile of LFT data
  Peter Lane, 22 February 2012, based on more general code by Shi-Tao Yeh (GSK)";

%let dir = %str(C:/book/Fig2_5/);
* Point to dataset locations (sheet of the Excel workbook);
libname xlab excel "&dir.lab.xls" access=readonly;
%let lab=xlab.'Sheet1$'n;
libname xmstone excel "&dir.mstone.xls" access=readonly;
%let mstone=xmstone.'Sheet1$'n;
filename psname "&dir.Fig2_5.ps";

* Define macro to draw panel for one patient;
%macro profile(pid=, gcat=, panel=);

  * Access information on specified patient in LAB database;
  data hit;
    set &lab;
	* Calculate LFT measurement in ULN units;
    uln = round(value/nr_hi, 0.01);
	* Confine attention to measurements on specified patient;
    if parm_c in ('ALKP', 'ALT', 'AST', 'TBL') and  pid="&pid";
	* Set up global variables with information for panel title;
    call symput('rand', substr(rand,6,1));
    call symput('race', compress(race));
    call symput('sex', compress(sex));
    call symput('age', compress(put(age,3.)));
    keep parm_c pid uln relt_fd rand sex age race;
  run;

  * Access start and stop times for specified patient from MSTONE database;
  data dur (keep=pid dur relt_fd);
    set &mstone;
    relt_fd = ltdat-ftdat+1;
    dur = -0.5;
    if pid="&pid";
  run;
  * Add start-time for this patient;
  data dur;
    set dur;
    output;
    relt_fd = 0;
    output;
  run;

  * Merge and sort the two datasets;
  data hit;
    set &lab;
    set hit dur;
    if parm_c='ALKP' then alkp = uln;
    else if parm_c='ALT' then alt = uln;
    else if parm_c='AST' then ast = uln;
    else if parm_c='TBL' then tbl = uln;
    st_days = dur;
  run;
  proc sort;
    by relt_fd;

  * Specify graphical details for this panel;
  * Increase font-size to allow for reduction putting four panels per page;
  title1 j=c h=18pt "Patient: &pid &race &sex Age: &age; Drug: &rand";
  symbol1 interpol=join font=orfont value=W height=10pt color=blue line=1 width=20;
  symbol2 interpol=join font=orfont value=E height=10pt color=orange line=1 width=20;
  symbol3 interpol=join font=orfont value=C height=10pt color=purple line=1 width=20;
  symbol4 interpol=join font=orfont value=D height=10pt color=green line=1 width=20;
  symbol5 interpol=join color=red line=1 width=40;
  axis1 order=(-50 to 200 by 25) label=(height=1pt ' ') width=1 value=(height=18pt);
  axis2 order=(-0.5 to 5 by 0.5) width=1 label=(height=18pt a=90 'LFT(/ULN)')
    value=(height=18pt tick=1 ' ');
  * Plot points and lines for each LFT, and line for duration of treatment;
  * Add reference lines in green, 50%-dashed;
  proc gplot data=hit gout=&gcat;
    plot alt*relt_fd=1 ast*relt_fd=2 alkp*relt_fd=3 tbl*relt_fd=4 st_days*relt_fd=5 /
      overlay name="&panel" cvref=green vref=0 1 2 3 lvref=2 vaxis=axis2 haxis=axis1;
  run;
%mend;
run;

* Set graphics details required for each panel;
goptions reset=all device=ps600c gsfname=psname gsfmode=replace goutmode=replace
  ftext=hwpsl009 vsize=4in hsize=9in;

* Create panels for each patient, appending to a temporary catalog "profile";
%profile(pid=1001, gcat=profile, panel=p1);
goptions goutmode=append;
%profile(pid=2002, gcat=profile, panel=p2);
%profile(pid=3003, gcat=profile, panel=p3);
%profile(pid=4004, gcat=profile, panel=p4);

goptions reset=all device=ps600c gsfname=psname gsfmode=replace
  ftext=hwpsl009 rotate=landscape vorigin=1in horigin=1in vsize=6in hsize=9in;
*title1 j=c h=14pt 'LFT Patient Profiles';
footnote1 h=12pt j=l 
  'The Clinical Concern Level (CCL) for TBL and ALKP is 2xULN (Upper Limit of Normal)';
footnote2 h=12pt j=l 
  'The CCL is usually 3xULN for ALT and AST, though it may be higher in some therapeutic areas';

* Draw legend explicitly;
proc gslide gout=profile name='gen';
  note move=(17,8)pct h=8pt f=orfont c=blue 'W ' f=hwpsl009 c=black h=10pt 'ALT';
  note move=(28,8)pct h=8pt f=orfont c=orange 'E ' f=hwpsl009 c=black h=10pt 'AST';
  note move=(40,8)pct h=8pt f=orfont c=purple 'C ' f=hwpsl009 c=black h=10pt 'ALKP';
  note move=(54,8)pct h=8pt f=orfont c=green 'D ' f=hwpsl009 c=black h=10pt 'TBL';
  note move=(68,10)pct h=60pt c=red '_' move=(75,8)pct c=black h=12pt 'Study Days';
run;

* Define template for arrangement of four panels and general information;
proc greplay tc=profile nofs;
  %let x1 = 0; %let x2 = 50; %let x3 = 100;
  %let y1 = 90; %let y2 = 50; %let y3 = 10;
  tdef proftemp
  1/ ulx=&x1 uly=&y1 urx=&x2 ury=&y1 llx=&x1 lly=&y2 lrx=&x2 lry=&y2
  2/ ulx=&x2 uly=&y1 urx=&x3 ury=&y1 llx=&x2 lly=&y2 lrx=&x3 lry=&y2
  3/ ulx=&x1 uly=&y2 urx=&x2 ury=&y2 llx=&x1 lly=&y3 lrx=&x2 lry=&y3
  4/ ulx=&x2 uly=&y2 urx=&x3 ury=&y2 llx=&x2 lly=&y3 lrx=&x3 lry=&y3
  5/ ulx=&x1 uly=100 urx=100 ury=100 llx=&x1 lly=0   lrx=100 lry=0;
run;

* Draw the display;
proc greplay igout=profile gout=profile tc=profile nofs;
  template proftemp;
  treplay 1:p1 2:p2 3:p3 4:p4 5:gen;
run;
quit;
